"""Thread wrappers for OpenFOAM"""
from threading import Thread,Lock
from popen2 import Popen4
from time import time
from resource import getrusage,getpagesize,RUSAGE_SELF,RUSAGE_CHILDREN

from PyFoam.Basics.LineReader import LineReader

class FoamThread(Thread):
    """Thread running an OpenFOAM command

    The output of the command can be accessed in a thread-safe manner,
    line by line

    Designed to be used by the BasicRunner-class"""
    
    def __init__(self,cmdline):
        """cmdline - Command line of the OpenFOAM command"""
        Thread.__init__(self)
        self.cmdline=cmdline
        self.output=None
        self.reader=LineReader()

        self.stateLock=Lock()
        self.setState(False)

        self.status=None

        self.lineLock=Lock()
        self.line=""

        self.resStart=None
        self.resEnd=None

        self.timeStart=None
        self.timeEnd=None
        
        self.stateLock.acquire()

    def run(self):
        """start the command"""
        # print "Starting ",self.cmdline
        self.resStart=getrusage(RUSAGE_CHILDREN)
        self.timeStart=time()
        
        run=Popen4(self.cmdline)
        self.output=run.fromchild

        self.hasSomethingToSay=True
        self.stateLock.release()

        self.status=run.wait()

        self.resEnd=getrusage(RUSAGE_CHILDREN)
        self.timeEnd=time()

        # print "Returned",self.status

    def read(self):
        """read another line from the output"""
        self.setState(self.reader.read(self.output))
        self.lineLock.acquire()
        self.line=self.reader.line
        self.lineLock.release()

    def getLine(self):
        """gets the last line from the output"""
        self.lineLock.acquire()
        val=self.line
        self.lineLock.release()

        return val
    
    def setState(self,state):
        """sets the state of the thread (is there any more output)"""
        self.stateLock.acquire()
        self.hasSomethingToSay=state
        #        print "Set: ",state
        self.stateLock.release()
        
    def check(self):
        """@return: False if there is no more output of the command"""
        self.stateLock.acquire()
        state=self.hasSomethingToSay
        #        print "Get: ",state
        self.stateLock.release()

        return state 
    
    def cpuTime(self):
        """@return: number of seconds CPU-Time used"""
        return self.cpuUserTime()+self.cpuSystemTime()
    
    def cpuUserTime(self):
        """@return: number of seconds CPU-Time used in user mode"""
        if self.resStart==None or self.resEnd==None:
            return 0
        else:
            return self.resEnd.ru_utime-self.resStart.ru_utime

    def cpuSystemTime(self):
        """@return: number of seconds CPU-Time used in system mode"""
        if self.resStart==None or self.resEnd==None:
            return 0
        else:
            return self.resEnd.ru_stime-self.resStart.ru_stime

    def usedMemory(self):
        """@return: maximum resident set size"""
        if self.resStart==None or self.resEnd==None:
            return 0
        else:
            return getpagesize()*(self.resEnd.ru_maxrss-self.resStart.ru_maxrss)

    def wallTime(self):
        """@return: the wall-clock-time needed by the process"""
        if self.timeStart==None or self.timeEnd==None:
            return 0
        else:
            return self.timeEnd-self.timeStart
        
