"""Basis for the handling of OpenFOAM-files"""

import os,re
from os import path
from tempfile import mktemp


from PyFoam.Basics.Utilities import Utilities
from PyFoam.Basics.LineReader import LineReader

class FileBasis(Utilities):
    """ Base class for the other OpenFOAM--file-classes"""
    
    removedString="//PyFoamRemoved: "
    """Comment for lines that were overwritten by PyFoam-routines"""
    
    addedString="//PyFoamAdded"
    """Comment for lines that were added by PyFoam-routines"""
    
    def __init__(self,name):
        """ name - Name of the file"""
        self.name = path.abspath(name)
        self.fh=None
     
    def openFile(self):
        """opens the file. To be overloaded by derived classes"""
        self.fh=open(self.name,"r")

    def closeFile(self):
        """ closes the file"""
        self.fh.close()
        self.fh=None

    def makeTemp(self):
        """creates a temporary file"""
        fn=mktemp(dir=path.dirname(self.name))
        fh=open(fn,"w")

        return fh,fn

    def goTo(self,l,s,out=None,echoLast=False,stop=None):
        """Read lines until a token is found

        @param l: a LineReader object
        @param s: the string to look for
        @param out: filehandle to echo the lines to
        @param stop: pattern that indicates that exp will never be found (only passed through to goMatch)
        @param echoLast: echo the line with the string"""
        exp=re.compile("( |^)"+s+"( |$)")
        m=self.goMatch(l,exp,out=out,stop=stop)
        if out!=None and echoLast:
            out.write(l.line+"\n")
            
    def goMatch(self,l,exp,out=None,stop=None):
        """Read lines until a regular expression is matched

        @param l: a LineReader object
        @param exp: the expression to look for
        @param out: filehandle to echo the lines to
        @param stop: pattern that indicates that exp will never be found
        @return: match-object if exp is found, the line if stop is found and None if the end of the file is reached"""        
        while l.read(self.fh):
            m=exp.match(l.line)
            if m!=None:
                return m
            elif stop!=None:
                if stop.match(l.line):
                    return l.line
            if out!=None:
                out.write(l.line+"\n")

        return None
    
    def copyRest(self,l,out):
        """Copy the rest of the file
        
        @param l: a LineReader object
        @param out: filehandle to echo the lines to"""        
        while l.read(self.fh):
            out.write(l.line+"\n")

    def purgeFile(self):
        """Undo all the manipulations done by PyFOAM

        Goes through the file and removes all lines that were added"""
        rmExp= re.compile("^"+self.removedString+"(.*)$")
        addExp=re.compile("^(.*)"+self.addedString+"$")
        
        l=LineReader()
        self.openFile()

        (fh,fn)=self.makeTemp()

        while l.read(self.fh):
            toPrint=l.line

            m=addExp.match(l.line)
            if m!=None:
                continue
            
            m=rmExp.match(l.line)
            if m!=None:
                toPrint=m.group(1)
                
            fh.write(toPrint+"\n")
        
        self.closeFile()
        fh.close()
        os.rename(fn,self.name)

class FileBasisBackup(FileBasis):
    """A file with a backup-copy"""

    def __init__(self,name,backup=False):
        """@param name: The name of the parameter file
        @type name: str
        @param backup: create a backup-copy of the file
        @type backup: boolean"""
        
        FileBasis.__init__(self,name)

        if backup:
            self.backupName=self.name+".backup"
            self.execute("cp "+self.name+" "+self.backupName)
        else:
            self.backupName=None

    def restore(self):
        """if a backup-copy was made the file is restored from this"""
        if self.backupName!=None:
            self.execute("cp "+self.backupName+" "+self.name)
    
