"""Working with a solution directory"""

from PyFoam.Basics.Utilities import Utilities
from PyFoam.Basics.BasicFile import BasicFile

from os import listdir,path,mkdir
import os

class SolutionDirectory(Utilities):
    """Represents a solution directory

    In the solution directory subdirectories whose names are numbers
    are assumed to be solutions for a specific time-step

    A sub-directory (called the Archive) is created to which solution
    data is copied"""
    
    def __init__(self,name,archive="ArchiveDir"):
        """@param name: Name of the solution directory
        @param archive: name of the directory where the lastToArchive-method
        should copy files, if None no archive is created"""
        self.name=name
        self.archive=None
        if archive!=None:
            self.archive=path.join(name,archive)
            if not path.exists(self.archive):
                mkdir(self.archive)

        self.backups=[]
        
        self.reread()

        self.essential=[self.systemDir(),self.constantDir(),self.initialDir()]

    def addToClone(self,name):
        """add directory to the list that is needed to clone this case
        @param name: name of the subdirectory (the case directory is prepended)"""
        self.essential.append(path.join(self.name,name))

    def cloneCase(self,name):
        """create a clone of this case directory. Remove the target directory, if it already exists

        @param name: Name of the new case directory
        @rtype: L{SolutionDirectory} or correct subclass
        @return: The target directory"""

        if path.exists(name):
            self.execute("rm -r "+name)
        mkdir(name)
        for d in self.essential:
            self.execute("cp -r "+d+" "+name)

        return self.__class__(name)
    
    def reread(self):
        """Rescan the directory for the time directories"""
        self.times=[]
        self.first=None
        self.last=None
        
        for f in listdir(self.name):
            try:
                val=float(f)
                self.times.append(f)
                if self.first==None:
                    self.first=f
                else:
                    if float(f)<float(self.first):
                        self.first=f
                if self.last==None:
                    self.last=f
                else:
                    if float(f)>float(self.last):
                        self.last=f
                        
            except ValueError:
                pass

    def addBackup(self,pth):
        """add file to list of files that are to be copied to the
        archive"""
        self.backups.append(path.join(self.name,pth))

    def getLast(self):
        """@return: the last time for which a solution exists
        @rtype: str"""
        self.reread()
        return self.last
    
    def lastToArchive(self,name):
        """copy the last solution (plus the backup-files to the
        archive)

        @param name: name of the sub-directory in the archive"""
        if self.archive==None:
            print "Warning: nor Archive-directory"
            return
        
        self.reread()
        fname=path.join(self.archive,name)
        if path.exists(fname):
            self.execute("rm -r "+fname)
        mkdir(fname)
        self.execute("cp -r "+path.join(self.name,self.last)+" "+fname)
        for f in self.backups:
            self.execute("cp -r "+f+" "+fname)
            
    def clearResults(self,after=None):
        """remove all time-directories after a certain time. If not time ist
        set the initial time is used"""

        self.reread()
        
        if after==None:
            time=float(self.first)
        else:
            time=float(after)
            
        for f in self.times:
            if float(f)>time:
                self.execute("rm -r "+path.join(self.name,f))
                
    def initialDir(self):
        """@return: the name of the first time-directory (==initial
        conditions
        @rtype: str"""
        return path.join(self.name,self.first)

    def latestDir(self):
        """@return: the name of the first last-directory (==simulation
        results)
        @rtype: str"""
        return path.join(self.name,self.getLast())

    def constantDir(self):
        """@return: the name of the C{constant}-directory
        @rtype: str"""
        return path.join(self.name,"constant")

    def systemDir(self):
        """@return: the name of the C{system}-directory
        @rtype: str"""
        return path.join(self.name,"system")

    def controlDict(self):
        """@return: the name of the C{controlDict}
        @rtype: str"""
        return path.join(self.systemDir(),"controlDict")

    def polyMeshDir(self):
        """@return: the name of the C{polyMesh}
        @rtype: str"""
        return path.join(self.constantDir(),"polyMesh")

    def blockMesh(self):
        """@return: the name of the C{blockMeshDict} if it exists. Returns
        an empty string if it doesn't
        @rtype: str"""
        p=path.join(self.polyMeshDir(),"blockMeshDict")
        if path.exists(p):
            return p
        else:
            return ""
        
    def makeFile(self,name):
        """create a file in the solution directory and return a
        corresponding BasicFile-object

        @param name: Name of the file
        @rtype: L{BasicFile}"""
        return BasicFile(path.join(self.name,name))

class ChemkinSolutionDirectory(SolutionDirectory):
    """Solution directory with a directory for the Chemkin-files"""

    chemkinName = "chemkin"
    
    def __init__(self,name,archive="ArchiveDir"):
        SolutionDirectory.__init__(self,name,archive=archive)

        self.addToClone(self.chemkinName)

    def chemkinDir(self):
        """@rtype: str
        @return: The directory with the Chemkin-Files"""

        return path.join(self.name,self.chemkinName)
    
