""" Working with solutions """

import re,os
import gzip
from os import path
from tempfile import mktemp

from PyFoam.Basics.LineReader import LineReader
from FileBasis import FileBasis

class SolutionFile(FileBasis):
    """ Solution data file

        Represents a file with the solution data for one
        OpenFOAM-field at one point of time

        Currently this can only handle uniform field values"""
    
    def __init__(self,directory,name):
        """ directory - name of the directory containing the solutions
        for a spcific time
        
        name - name of the field. If the field is zipped the .gz is
        appended"""
        
        FileBasis.__init__(self,path.abspath(path.join(directory,name)))
        if path.exists(self.name):
            self.zipped=False
        else:
            self.zipped=True
            self.name+=".gz"
            
        self.fh=None
        
    def openFile(self):
        """opens the file (if zipped, the file is written zipped)"""
        
        if self.zipped:
            self.fh=gzip.open(self.name,"r")
        else:
            self.fh=open(self.name,"r")

    def makeTemp(self):
        """creates a temporary file (if the original is zipped, this
        is zipped too"""
        fn=mktemp(dir=path.dirname(self.name))
        if self.zipped:
            fh=gzip.open(fn,"w")
        else:
            fh=open(fn,"w")

        return fh,fn
    
    def internalPattern(self):
        """pattern for internal fields"""
        return re.compile("^internalField +uniform +(.+);")

    def valuePattern(self):
        """pattern for values"""
        return re.compile("value +uniform +(.+);")

    def stopPattern(self):
        """pattern that ends a boundary"""
        return re.compile("^\b*}")

    def readBoundary(self,name):
        """read the value at a boundary

        name - the name of the boundary patch"""
        exp=self.valuePattern()
        erg=""
        
        l=LineReader()
        self.openFile()

        self.goTo(l,"boundaryField")
        self.goTo(l,name)
        
        m=self.goMatch(l,exp)
        if m!=None:
            erg=m.group(1)
        
        self.closeFile()
        return erg

    def replaceBoundary(self,name,newval):
        """write the value at a boundary

        @param name: the name of the boundary patch
        @param newval: the new value"""
        exp=self.valuePattern()

        l=LineReader()
        self.openFile()

        fh,fn=self.makeTemp()

        self.goTo(l,"boundaryField",out=fh,echoLast=True)
        self.goTo(l,name,out=fh,echoLast=True)

        m=self.goMatch(l,exp,out=fh,stop=self.stopPattern())
        
        if m!=None:
            if type(m)==str:
                fh.write("value uniform "+str(newval)+"; "+self.addedString+"\n")
                fh.write(l.line+"\n")
            else:
                fh.write(self.removedString+l.line+"\n")
                fh.write("value uniform "+str(newval)+"; "+self.addedString+"\n")
        else:
            fh.write(l.line+"\n")

        self.copyRest(l,fh)

        self.closeFile()
        fh.close()
        os.rename(fn,self.name)
        
    def readInternal(self):
        """read the value of the internal field"""
        exp=self.internalPattern()
        erg=""
        
        l=LineReader()
        self.openFile()

        while l.read(self.fh):
            m=exp.match(l.line)
            if m!=None:
                erg=m.group(1)
                break

        self.closeFile()
        return erg
    
    def replaceInternal(self,newval):
        """overwrite the value of the internal field

        newval - the new value"""        
        exp=self.internalPattern()
 
        l=LineReader()
        self.openFile()

        fh,fn=self.makeTemp()

        m=self.goMatch(l,exp,out=fh)
        
        if m!=None:
            fh.write(self.removedString+l.line+"\n")
            fh.write("internalField uniform "+str(newval)+"; "+self.addedString+"\n")
        else:
            fh.write(l.line+"\n")

        self.copyRest(l,fh)

        self.closeFile()
        fh.close()
        os.rename(fn,self.name)
      
